﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModel;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModel;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels;
using SushiBarDataModels.Models;
using System.Numerics;

namespace SushiBarBusinessLogic
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;


        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}, ShopName:{ ShopName}", model?.Id, model?.ShopName);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadList. Id:{ Id}, ShopName:{ ShopName}", model.Id, model.ShopName);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Вставка в хранилище прервана");
                return false;
            }
            return true;
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Обновление прервано");
                return false;
            }
            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model);
            if (_shopStorage?.Delete(model) == null)
            {
                _logger.LogWarning("Удаление прервано");
                return false;
            }
            return true;
        }

        public bool AddSushiInShop(ShopSearchModel model, ISushiModel sushi, int count)
        {
            if (model == null)
                throw new ArgumentNullException("Не найден такой магазин... \n", nameof(model));
            if (count <= 0)
                throw new ArgumentException("Количество суши должно быть больше нуля, ало \n", nameof(count));
            _logger.LogInformation("Добавлены суши в магазин: {ShopName}.Id:{ Id}", model.ShopName, model.Id);

            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("Не добавлено, магазин не найден с таким названием");
                return false;
            }
            var countSushis = element.ShopSushis.Select(x => x.Value.Item2).Sum();
            if (element.MaxCountSushis - countSushis < count)
            {
                _logger.LogWarning("В магазине не хватает места");
                throw new ArgumentOutOfRangeException("Не зватает места в магазине, увы... кушать меньше надо ", nameof(countSushis));
            }
            if (element.ShopSushis.TryGetValue(sushi.Id, out var samesushi))
            {
                element.ShopSushis[sushi.Id] = (sushi, samesushi.Item2 + count);
                _logger.LogInformation("Same sushi found by supply. Added {0} of {1} in {2} shop", count, sushi.SushiName, element.ShopName);
            }
            else
            {
                element.ShopSushis[sushi.Id] = (sushi, count);
                _logger.LogInformation("New sushi added by supply. Added {0} of {1} in {2} shop", count, sushi.SushiName, element.ShopName);
            }

            _shopStorage.Update(new()
            {
                Id = element.Id,
                ShopName = element.ShopName,
                Address = element.Address,
                DateOpening = element.DateOpening,
                ShopSushis = element.ShopSushis,
                MaxCountSushis = element.MaxCountSushis,
            });

            return true;
        }

        public bool SellSushis(ISushiModel sushi, int count)
        {
            if (sushi == null)
            {
                throw new ArgumentNullException("Не найдены такие суши, их нет в магазинах... \n", nameof(sushi));
            }
            if (count <= 0)
            {
                throw new ArgumentNullException("Количество суши должно быть больше нуля! алло!\n", nameof(count));
            }
            if (_shopStorage.SellSushis(sushi, count))
            {
                _logger.LogInformation("Selling sucsess");
                return true;
            }
            _logger.LogInformation("Selling failed");
            return false;
        }


        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
                throw new ArgumentNullException($"{nameof(model)} является null");
            if (!withParams) return;
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет такого магазина ", nameof(model.ShopName));
            }
            _logger.LogInformation("Shop. ShopName:{ShopName}.Address:{ Address}. Id:{ Id}",
                model.ShopName, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = model.ShopName,
            });
            if (element != null && element.Id != model.Id && element.ShopName == model.ShopName)
            {
                throw new InvalidOperationException("Такой магазин с таким названием уже есть");
            }
        }
    }
}
